Disk encryption
===============

In order to mount encrypted disk, Sabakan provides `sabakan-cryptsetup` as follows.

* Sabakan keeps an `encryption key` for each disk which is specified by `sabakan-cryptsetup` argument.

    Each node generates a pair of `64 byte` random keys. One is for [cryptsetup][], and another is to encrypt
    the other by [one-time pad][]. The node keeps this latter key in the first sector of the metadata partition.

    The `encryption key` is stored `sabakan`. Hence, the node cannot decrypt its own storage data without `sabakan`, and vice versa.

    `sabakan-cryptsetup` requests `/api/v1/crypts` to manage keys. See details [API](api.md).

* Operators need to setup RAID, format filesystem, and mount disk after decrypt disks.

    Operators needs to prepare own scripts or systemd units to mount disks.

## Crypt specs

`sabakan-cryptsetup` uses [cryptsetup][] in plain mode, therefore, it is mostly raw dm-crypt.

| Parameter    | Value
| ---------    | -----
| Cipher       | AES
| Chain Mode   | XTS
| IV generator | plain64
| Key bits     | 512 (256 for AES, 256 for XTS)

`sabakan-cryptsetup` writes/reads from the disk as metadata. Space for metadata keeps `2 MiB` by `--offset` option of the `cryptsetup`.

| Name           | Size(byte) | Description
| ----           | ---------- | -----------
| `magic`        | 8          | A magic number to detect if disk is encrypted by `sabakan-crypsetup`.
| `one-time pad` | 64         | An one-time pad as a pair of XOR. It is generated by `crypto/rand` of Go.
| `ID`           | 16         | An unique identifier like an UUID to detect if `encryption key` is registered on the `sabakan`. It is generated by `crypto/rand` of Go.

## Procedure of the disk encryption and decryption

1. `sabakan-cryptsetup` detect storage devices on the `/dev/disk/by-path` using a glob which is specified as an argument.
2. Read `one-time pad` and `ID`, and send HTTP request to `sabakan` whether an `encryption key` and `ID` of its disk are registered. If not, `sabakan-cryptsetup` re-encrypts the disk.
    1. Generate `one-time pad`, a pair of `key` and `ID`.
    2. Write `magic`, `one-time pad` and `ID` to metadata partition.
    3. Register XOR value between `one-time pad` as an `encryption key` to `sabakan` with `ID`.
3. Execute `cryptsetup` to create `/dev/mapper/crypt-BYPATH` of each disk as decryption. `BYPATH` is the name of the device, shown in `/dev/disk/by-path`.
4. Setup md device and/or initialize filesystem.
5. Mount filesystem.
6. Ready for apps!

To deploy above procedure for nodes, Operator needs to prepare systemd unit files by Ignition. For example,

- `sabakan-cryptsetup.service` ... An oneshot service executes `sabakan-cryptsetup` to create a device mapper.
- `setup-disk.service` ... Create a filesystem, then mount a device mapper.

[dm-crypt]: https://gitlab.com/cryptsetup/cryptsetup/wikis/DMCrypt
[one-time pad]: https://en.wikipedia.org/wiki/One-time_pad
[cryptsetup]: https://gitlab.com/cryptsetup/cryptsetup/wikis/home
